/*
Copyright (c) 2007 Health Market Science, Inc.

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

You can contact Health Market Science at info@healthmarketscience.com
or at the following address:

Health Market Science
2700 Horizon Drive
Suite 200
King of Prussia, PA 19406
*/

package com.healthmarketscience.common.util;

import java.io.IOException;

/**
 * Simple extension of AppendableExt that swallows IOExceptions under the
 * assumption that it will generally be used with Appendables which do not
 * actually throw any IOExceptions (i.e. StringBuilder).  Default constructors
 * will use a StringBuilder for the underlying Appendable.
 * <p>
 * In the event that an underlying Appendable is used which does actually
 * thrown IOExceptions, they will be trapped, and at any time the last
 * IOException can be obtained using {@link #getIOException}.
 * </p>
 * <p>
 * This class is not thread-safe.
 * </p>
 */
public class StringAppendableExt extends AppendableExt
{

  /** 
   * The last IOException thrown by an append call
   */
  private IOException _ioException;
    
  /** 
   * Initialize a new StringAppendableExt based on a StringBuilder.
   */
  public StringAppendableExt() {
    this(new StringBuilder(), null);
  }

  /** 
   * Initialize a new StringAppendableExt based on a StringBuilder, with a
   * specified initial capacity.
   * 
   * @param capacity the initial capacity for the StringBuilder.
   */
  public StringAppendableExt(int capacity) {
    this(capacity, null);
  }

  /** 
   * Initialize a new StringAppendableExt based on a StringBuilder, with a
   * specified initial capacity and given context.
   * 
   * @param capacity the initial capacity for the StringBuilder.
   * @param context initial append context
   */
  public StringAppendableExt(int size, Object context) {
    this(new StringBuilder(size), context);
  }

  /** 
   * Initialize a new StringAppendableExt based on the given Appendable.
   * 
   * @param app initial underlying Appendable
   */
  public StringAppendableExt(Appendable app) {
    this(app, null);
  }

  /** 
   * Initialize a new StringAppendableExt based on the given Appendable and
   * context.
   * 
   * @param app initial underlying Appendable
   * @param context initial append context
   */
  public StringAppendableExt(Appendable app, Object context) {
    super(app, context);
  }

  @Override
  public StringAppendableExt append(char c)
  {
    try {
      super.append(c);
    } catch(IOException e) {
      _ioException = e;
    }
    return this;
  }
  
  @Override
  public StringAppendableExt append(CharSequence s)
  {
    try {
      super.append(s);
    } catch(IOException e) {
      _ioException = e;
    }
    return this;
  }
  
  @Override
  public StringAppendableExt append(CharSequence s, int start, int end)
  {
    try {
      super.append(s, start, end);
    } catch(IOException e) {
      _ioException = e;
    }
    return this;
  }

  @Override
  public StringAppendableExt append(Appendee a)
  {
    try {
      super.append(a);
    } catch(IOException e) {
      _ioException = e;
    }
    return this;
  }

  @Override
  public StringAppendableExt append(Object o)
  {
    try {
      super.append(o);
    } catch(IOException e) {
      _ioException = e;
    }
    return this;
  }

  @Override
  public StringAppendableExt append(Iterable<?> iable, Object delimiter)
  {
    try {
      super.append(iable, delimiter);
    } catch(IOException e) {
      _ioException = e;
    }
    return this;
  }

  /**
   * Get the last IOException thrown by an append call, or {@code null} if 
   * no IOException was caught.
   *
   * @return the last IOException thrown by an append call, or {@code null}
   *         if no IOException was caught.
   */
  public IOException getIOException() {
    return _ioException;
  }
  
}
