/*
Copyright (c) 2007 Health Market Science, Inc.

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

You can contact Health Market Science at info@healthmarketscience.com
or at the following address:

Health Market Science
2700 Horizon Drive
Suite 200
King of Prussia, PA 19406
*/

package com.healthmarketscience.common.util;

import java.io.Serializable;
import java.util.AbstractCollection;
import java.util.AbstractList;
import java.util.Collection;
import java.util.Iterator;
import java.util.List;
import java.util.RandomAccess;

import org.apache.commons.lang.builder.EqualsBuilder;
import org.apache.commons.lang.builder.HashCodeBuilder;

/**
 * Simple class for holding a single typed object (useful for returning
 * additional objects from a method call).  For convenience, a Tuple can act
 * like an unmodifiable List of Object of a fixed size (the number of
 * elements in the tuple).
 *
 * @author James Ahlborn
 */
public class Tuple1<ObjType0>
  extends AbstractList<Object>
  implements Serializable, List<Object>, RandomAccess, Cloneable
{

  private static final long serialVersionUID = 20071230;
  
  /** 
   * Helper array to get direct indexed access to the tuple values.
   */
  static final Accessor[] ACCESSORS1 = {
    new Accessor() {
      @Override
      public Object get(Tuple1<?> tuple) { return tuple.get0(); }
    }
  };
  
  /** 
   * The value stored in this tuple.
   */
  private ObjType0 _obj0;
  
  /**
   * Initializes a new Tuple1 with a null object.
   */
  public Tuple1() {
    this(null);
  }

  /**
   * Initializes a new Tuple1 with the provided object. You may want to use
   * {@link #create} instead, since it infers the type parameter based on
   * its argument.
   *
   * @param obj the object to store in this Tuple1.
   */
  public Tuple1(ObjType0 obj) {
    _obj0 = obj;
  }

  /**
   * Creates a new Tuple1 with the provided object.
   *
   * @param obj0 the object to store in this Tuple1.
   * @return a new Tuple1 with the provided object.
   */
  public static <InObjType0> Tuple1<InObjType0> create(InObjType0 obj0)
  {
    return new Tuple1<InObjType0>(obj0);
  }

  /**
   * Returns an Iterable adapter that returns an Iterator that iterates over
   * element 0 of the tuples within the given Iterable/Iterator.  The returned
   * Iterator supports element removal if the given Iterable supports element 
   * removal. 
   *
   * @return an Iterable adapter that returns an Iterator that iterates
   *         over element 0 of the Tuples within the given Iterable/Iterator.
   *         
   */
  public static <InObjType0> Iterable<InObjType0> iterable0(
      final Iterable<? extends Tuple1<InObjType0>> iable)
  {
    return new IterableIterator<InObjType0>()
      {
        private final Iterator<? extends Tuple1<InObjType0>> _iter =
          iable.iterator();
        public boolean hasNext() {
          return _iter.hasNext();
        }
        public InObjType0 next() {
          return _iter.next().get0();
        }
        public void remove() {
          _iter.remove();
        }
      };
  }

  /** 
   * Returns a Collection adapter for element 0 of the Tuples of the given
   * Collection.  The returned Collection supports element removal if the given
   * Collection supports element removal, but does not support element 
   * addition.
   *
   * @return a Collection adapter for element 0 of the Tuples of the given
   *         Collection.  
   */
  public static <InObjType0> Collection<InObjType0> collection0(
      final Collection<? extends Tuple1<InObjType0>> col)
  {
    return new AbstractCollection<InObjType0>()
      {
        private final Collection<? extends Tuple1<InObjType0>> _col = col;
        @Override
        public Iterator<InObjType0> iterator() {
          return iterable0(_col).iterator();
        }
        @Override
        public int size() { return _col.size(); }
      };
  }
  
  /**
   * Get the object of this tuple.
   *
   * @return the object of this tuple.
   */
  public final ObjType0 get0() { return _obj0; }

  /**
   * Sets the object of this tuple.
   *
   * @param obj the object to store in this tuple.
   */
  public final void set0(ObjType0 obj) { _obj0 = obj; }

  /**
   * Gets the object value of this tuple as an Object array.
   *
   * @return the object value of this tuple as an Object array.
   */
  public Object[] get() { return new Object[]{get0()}; }

  /** 
   * Get the value of this tuple identified by index, which must always be
   * 0, since this tuple class stores exactly one object.
   *
   * @param index the index of the object to retrieve.  Must be 0.
   * @return the object in this tuple with the given index.
   */
  @Override
  public Object get(int index) { return ACCESSORS1[index].get(this); }

  /**
   * Sets the value of this tuple.
   *
   * @param obj0 the value to store in this tuple.
   */
  public final void set(ObjType0 obj0) { set0(obj0); }

  /** 
   * Get the size of this tuple, which is always 1.
   *
   * @return 1
   */
  @Override
  public int size() { return 1; }

  /** 
   * Makes a copy of this Tuple1.
   * 
   * @return a copy of this Tuple1.
   */
  @SuppressWarnings({"unchecked",
                     "PMD.CloneThrowsCloneNotSupportedException"})
  @Override
    //@SuppressWarnings("unchecked")
  public Tuple1<ObjType0> clone() {
    try {
      return (Tuple1<ObjType0>)super.clone();
    } catch(CloneNotSupportedException e) {
      // this shouldn't happen, since we are Cloneable
      throw new UnsupportedOperationException(e);
    }
  }

  /** 
   * Indicates whether the given Object is equal to this Tuple1.
   * @return 
   *  <li>{@code true} if the given Object refers to this Tuple1.</li>
   *  <li>{@code true} if the given Object is an instance of Tuple1 for which
   *      the contained object is equal to this Tuple1's object.</li>
   *  <li>{@code false} otherwise.</li>
   */
  @Override
  public boolean equals(Object o)
  {
    if(this == o) {
      return true;
    }
    if((o == null) || (o.getClass() != getClass())) {
      return false;
    }
    
    return new EqualsBuilder()
      .append(get0(), ((Tuple1)o).get0())
      .isEquals();
  }

  /** 
   * Returns a hash code for this Tuple1, based on the object it contains.
   *
   * @return a hash code for this Tuple1.
   */
  @Override
  public int hashCode()
  {
    return new HashCodeBuilder()
      .append(get0())
      .toHashCode();
  }

  /** 
   * Helper class to enable direct indexed access to the tuple values.
   */
  protected abstract static class Accessor {
    public abstract Object get(Tuple1<?> tuple);
  }
  
  /**
   * An Iterable whose iterator method returns itself.
   */
  protected static abstract class IterableIterator<Type>
    implements Iterable<Type>, Iterator<Type>
  {
    public Iterator<Type> iterator() {
      return this;
    }
  }


}
